/*********************************************************************
* Copyright (C) 2006-2015 by Roundtable Software Inc. ("Roundtable"),*
*  All Rights Reserved.                                              *
*                                                                    *
* Software distributed under the License is distributed on an "AS IS"*
* basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. You*
* should refer to the License for the specific language governing    *
* rights and limitations under the License.                          *
*                                                                    *
* Contributors:                                                      *
* nodeable limited (www.nodeable.io)                                 *
*********************************************************************/
/*------------------------------------------------------------------------
  File        : rtb_events.p
  Description : Roundtable published event handler
  Notes       :

    The procedure is run persistently in the Roundtable session and
    receives the published event as documented below. To intercept an
    invent, modify 'evRtbUserEvent' as needed to respond to the desired
    published event.
 ----------------------------------------------------------------------*/

/*
Roundtable Published Events and Parameters:

"addObjectBefore"
  Pcontext = ""
  Pother   = Object Name, Object Type, Product Module, Workspace ID
  Pok      = False will cancel

"addObject"
  Pcontext = rtb_object ROWID string value
  Pother   = Object Name, Object Type, Product Module, Workspace ID

"assignObjectBefore"
  Pother   = Object Name, Object Type, Product Module, Version, Workspace ID
  Pok      = False will cancel

"assignObject"
  Pcontext = rtb_object ROWID string value
  Pother   = Object Name, Object Type, Product Module, Workspace ID

"buildImportListBefore"
    Pcontext = blank
    Pother   = Target Workspace
    Pok      = False will cancel

"buildImportList"
    Pcontext = blank
    Pother   = Target Workspace

"buildSchemaListBefore"
    Pcontext = blank
    Pother   = Database Name
    Pok      = False will cancel

"buildSchemaList"
    Pcontext = blank
    Pother   = Database Name

"changeObjectShareStatusBefore"
  Pcontext = rtb_object ROWID string value
  Pother   = rtb_object RECID string value, Object Name, Old Status, New Status, WorkspacePath, Task Path
  Pok      = False will cancel

"changeObjectShareStatus"
  Pcontext = rtb_object ROWID string value
  Pother   = rtb_object RECID string value, Object Name, Old Status, New Status, WorkspacePath, Task Path

"changePropathBefore"
    Pcontext = rtb_object ROWID string value
    Pother   = "BEFORE-COMPILE" or "AFTER-COMPILE"

"changePropath"
  Pcontext = rtb_object ROWID string value
  Pother   = "BEFORE-COMPILE" or "AFTER-COMPILE"

"changeWorkspaceBefore"
  Pcontext = Desktop handle string value
  Pother   = From-Workspace, To-Workspace
  Pok      = False will cancel

"changeTask"
  Pother   = Task Number

"changeWorkspace"
  Pcontext = Desktop handle string value
  Pother   = Workspace ID

"checkinObjectBefore"
  Pcontext = rtb_object ROWID string value
  Pother   = Object Name, Object Type, Product Module, Workspace ID
  Pok      = False will cancel

"checkinObject"
  Pcontext = rtb_object ROWID string value
  Pother   = Object Name, Object Type, Product Module, Workspace ID

"checkoutObjectBefore"
  Pcontext = rtb_object ROWID string value
  Pother   = Object Name, Object Type, Product Module, Workspace ID, Level ("version","revision","patch")
  Pok      = False will cancel

"checkoutObject"
  Pcontext = rtb_object ROWID string value
  Pother   = Object Name, Object Type, Product Module, Workspace ID

"compileObjectBefore"
  Pcontext = rtb_object ROWID string value
  Pother   = Object Name, Object Type, Product Module, Workspace ID
  Pok      = False will cancel

"compileObject"
  Pcontext = rtb_object ROWID string value
  Pother   = Object Name, Object Type, Product Module, Workspace ID

"compileRecordCreate"
  Pcontext = Buffer handle of compile record

"completeTaskBefore"
  Pother   = Task Number
  Pok      = False will cancel

"completeTask"
  Pother   = Task Number

"createObjectVariantBefore"
  Pcontext = rtb_object ROWID string value
  Pother   = Object Name, Object Type, Current Product Module, New Produce Module, Workspace ID
  Pok      = False will cancel

"createObjectVariant"
  Pcontext = rtb_object ROWID string value
  Pother   = Object Name, Object Type, Product Module, Workspace ID

"createReleaseBefore"
  Pother   = Workspce ID
  Pok      = False will cancel

"createRelease"
  Pother   = Workspace ID, Release Number

"createSiteBefore"
  Pother   = Workspace ID, License Type, Site Code
  Pok      = False will cancel

"createSite"
  Pother   = Workspace ID, License Type, Site Code

"createTaskBefore"
  Pcontext = CHR(1)-delimited list of column/value pairs
  Pother   = Workspace ID
  Pok      = False will cancel

"createTask"
  Pcontext = rtb_task ROWID string value
  Pother   = Task Number

"deleteObjectBefore"
  Pcontext = rtb_object ROWID string value
  Pother   = Object Name, Object Type, Product Module, Workspace ID
  Pok      = False will cancel

"deleteObject"
  Pcontext = WIP version of the deleted object
  Pother   = Object Name, Object Type, Product Module, Workspace ID

"deleteTaskBefore"
  Pcontext = RowObjUpd buffer handle string value
  Pother   = Task Number
  Pok      = False will cancel

"deleteTask"
  Pcontext = rtb_task ROWID string value
  Pother   = Task Number

"loadPartnerBefore"
  Pcontext = Workspace ID
  Pother   = Workspace Path
  Pok      = False will cancel

"loadPartner"
  Pcontext = rtb_wspace ROWID string value
  Pother   = Object Name

"makeDeploymentBefore"
  Pcontext = rtb_site ROWID string value
  Pother   = rtb_deploy ROWID string value
  Pok      = False will cancel

"makeDeployment"
  Pcontext = rtb_site ROWID string value
  Pother   = rtb_deploy ROWID string value

"modifyTaskBefore"
  Pcontext = RowObjUpd buffer handle string value
  Pother   = Task Number
  Pok      = False will cancel

"modifyTask"
  Pcontext = rtb_task ROWID string value
  Pother   = Task Number

"moveToWebBefore"
  Pcontext = rtb_object ROWID string value
  Pother   = Object Name, Object Type, Pmod, Module
  Pok      = False will cancel

"moveToWeb"
  Pcontext = rtb_object ROWID string value
  Pother   = Object Name

"newTaskRow"
  Pcontext = RowObject buffer handle string value
  Pother   = Workspace ID

"processImportBefore"
  Pcontext = Workspace ID
  Pok      = False will cancel

"processImport"
  Pcontext = Workspace ID

"roundtableStartup"
  Pcontext = Tabletop procedure handle string value
  Pother   = Tabletop window handle string value

"roundtableShutdown"

"sessionStartup"
  Pcontext = Session ID string value
  Pok      = False will cancel

"sessionShutdown"

"updateSchemaBefore"
  Pother   = Workspace ID
  Pok      = False will cancel

"updateSchema"
  Pother   = Workspace ID

*/

SUBSCRIBE TO "evRtbUserEvent":U ANYWHERE.

PROCEDURE evRtbUserEvent :
/*------------------------------------------------------------------------------
  Purpose:     Roundtable Event Handler
  Parameters:  Pevent
               Pcontext
               Pother
               Pok
  Notes:       Common procedure for all events.
               RETURN a value from this procedure to return an error message
               to the caller.
------------------------------------------------------------------------------*/
  DEFINE INPUT  PARAMETER Pevent   AS CHARACTER.
  DEFINE INPUT  PARAMETER Pcontext AS CHARACTER.
  DEFINE INPUT  PARAMETER Pother   AS CHARACTER.
  DEFINE OUTPUT PARAMETER Pok      AS LOGICAL INITIAL YES.

   DEFINE VARIABLE cErrorMessage       AS CHARACTER  NO-UNDO INITIAL "".
    DEFINE VARIABLE cReturnMessage AS CHARACTER NO-UNDO.
    DEFINE VARIABLE hLoopHandle    AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hRtbEvent      AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hPreRtbEvent   AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hXMLConfMan    AS HANDLE    NO-UNDO.
    DEFINE VARIABLE hRTBDeskHandle AS HANDLE    NO-UNDO.
    DEFINE VARIABLE lDebugOn       AS LOGICAL   NO-UNDO INITIAL FALSE.

  /* Uncomment for debugging */

  /*
  MESSAGE
    "Event:   " Pevent SKIP
    "Context: " Pcontext SKIP
    "Other:   " Pother

    VIEW-AS ALERT-BOX
    BUTTONS OK-CANCEL
    TITLE "Roundtable Event"
    UPDATE Pok.
  */

        ASSIGN
            cErrorMessage = "":U.

        IF NOT CONNECTED("ICFDB":U) THEN
            RETURN.

        /* First walk the widget tree and get the handles */
        /* See if already running first */
        ASSIGN
            hLoopHandle = SESSION:FIRST-PROCEDURE.

        DO WHILE VALID-HANDLE(hLoopHandle):

            IF NOT VALID-HANDLE(hRtbEvent) AND
                ( hLoopHandle:FILE-NAME    = "rtb/prc/ryrtbevntp.p":U OR
                hLoopHandle:PRIVATE-DATA = "ryrtbevntp.p":U)
                THEN
                ASSIGN hRtbEvent = hLoopHandle.

            IF NOT VALID-HANDLE(hPreRtbEvent) AND
                ( hLoopHandle:FILE-NAME    = "rtb/prc/rtbpreevent.p":U OR
                hLoopHandle:PRIVATE-DATA = "rtbpreevent.p":U)
                THEN
                ASSIGN hPreRtbEvent = hLoopHandle.

            IF NOT VALID-HANDLE(hXMLConfMan) AND
                (R-INDEX(hLoopHandle:FILE-NAME,"afxmlcfgp.p":U) > 0 OR
                R-INDEX(hLoopHandle:FILE-NAME,"afxmlcfgp.r":U) > 0)
                THEN
                ASSIGN hXMLConfMan = hLoopHandle.

            IF  VALID-HANDLE(hRtbEvent) AND
                VALID-HANDLE(hPreRtbEvent) AND
                VALID-HANDLE(hXMLConfMan)
                THEN
                ASSIGN hLoopHandle = ?.
            ELSE
                ASSIGN hLoopHandle = hLoopHandle:NEXT-SIBLING.
        END.

        /***********************************************************************************/
        /* Pre-validation : Any code in this section is not bound to the Repository at all */
        /***********************************************************************************/
        IF NOT VALID-HANDLE(hPreRtbEvent) THEN
            RUN rtb/prc/rtbpreevent.p PERSISTENT SET hPreRtbEvent NO-ERROR.

        IF VALID-HANDLE(hPreRtbEvent) THEN
            RUN processPreEvent IN hPreRtbEvent
                (INPUT pEvent,
                INPUT pContext,
                INPUT pOther,
                OUTPUT cReturnMessage).
        IF cReturnMessage NE "":U THEN
        DO:
            IF cReturnMessage = "Cancelled":U THEN
                ASSIGN pOk = FALSE.
            ELSE
                ASSIGN pOk = TRUE.
            RETURN.
        END.

        /*****************/
        /* Standard Code */
        /*****************/

        IF  pEvent  NE "changeWorkspaceBefore" THEN
        DO:
            IF NOT VALID-HANDLE(hRtbEvent) THEN
            DO:
                IF SEARCH("rtb/prc/ryrtbevntp.p":U) NE ? OR
                    SEARCH("rtb/prc/ryrtbevntp.r":U) NE ? THEN
                    RUN rtb/prc/ryrtbevntp.p PERSISTENT SET hRtbEvent.

                /* Set the debug status */
                PUBLISH "evRtbCustSetDebug" (INPUT lDebugOn).
            END.
        END.

        IF  pEvent   = "changeWorkspaceBefore":U AND
            ENTRY(1,pOther) NE "":U THEN /* only if previous workspace valid - kill running plips */
        DO:
            IF VALID-HANDLE(hXMLConfMan) THEN
                APPLY "CLOSE":U TO hXMLConfMan.

            IF VALID-HANDLE(hRtbEvent) THEN
                RUN killPlip IN hRtbEvent.

            IF VALID-HANDLE(hPreRtbEvent) THEN
                RUN killPlip IN hPreRtbEvent.

            ASSIGN
                hXMLConfMan  = ?
                hRtbEvent    = ?
                hPreRtbEvent = ?.

            /* Remove the Progress Dynamics menu from the tabletop */
            PUBLISH "evRtbGetDeskHandle":U (OUTPUT hRTBDeskHandle).
            RUN rtb/prc/afrtbmenup.p (INPUT hRTBDeskHandle).
        END.

        IF pEvent = "changeWorkspace":U THEN
        DO:
            FIND FIRST rtb.rtb_wspace WHERE rtb_wspace.wspace-id = pOther NO-LOCK NO-ERROR.
            IF AVAILABLE rtb_wspace THEN
                IF INDEX(rtb_wspace.locked-by, "DCU":U) > 0 THEN
                DO:
                    ASSIGN
                        cErrorMessage = "The Dynamics environment has not been started. The workspace is locked : " + "~n" +
                                    "'" + rtb_wspace.locked-by + "'" + "~n":U + "~n":U +
                                   "Access to the workspace is limited as the integration hooks between RTB" + "~n":U +
                                   "and Dynamics are not enabled. " + "~n":U + "~n":U +
                                   "Wait until the DCU upgrade has been completed and re-select the workspace, " + "~n":U +
                                   "or contact your system administrator for further information.".

                    IF VALID-HANDLE(hRtbEvent) THEN
                        RUN killPlip IN hRtbEvent.

                    RETURN.
                END.

            /* Start the Dynamics environment  */
            IF VALID-HANDLE(hRtbEvent) THEN
                RUN icfstart.p /* NO-ERROR */.

            /* Add the Progress Dynamics menu to the tabletop */
            /* Set Grtb-proc-handle  */
            PUBLISH "evRtbGetDeskHandle":U (OUTPUT hRTBDeskHandle).
            RUN rtb/prc/afrtbmenup.p (INPUT hRTBDeskHandle) NO-ERROR.

        END.

        IF VALID-HANDLE(hRtbEvent) THEN
            RUN processEvent IN hRtbEvent
                (INPUT pEvent,
                INPUT pContext,
                INPUT pOther,
                OUTPUT cErrorMessage).

        IF cErrorMessage NE "":U THEN
            MESSAGE
                "RTB Error: " + cErrorMessage
                VIEW-AS ALERT-BOX
                INFORMATION
                BUTTONS OK.

        IF cErrorMessage NE "":U THEN
            ASSIGN pOk = FALSE.
        ELSE
            ASSIGN pOk = TRUE.

    RETURN.

END PROCEDURE.
