/*------------------------------------------------------------------------
    File        : rtb_events.p
    Description : Roundtable event handler

    Author(s)   :
    Created     :
    Notes       : Backwards-compatible with rtb_evnt.p customizations.
  ----------------------------------------------------------------------*/

/* ***************************  Definitions  ************************** */
DEFINE VARIABLE Muse-legacy AS LOGICAL NO-UNDO.

&SCOPED-DEFINE PRODUCT "Roundtable":U

/*
Roundtable User Events

rtb_events_cl.p will only run on the client when running Roundtable
in AppServer mode.  These events will be the only ones avaible to
the client.

rtb_events.p replaces rtb_evnt.p.  However, you can still use rtb_evnt.p.
Copy rtb_evnt.p to the root of your Roundtable installtion directory.  This
program will detect it's existence, translate the events, and pass them on
to your legacy event handler.  See the Main-Block of this procedure.

"changeWorkspaceBefore"
  Pcontext = Desktop handle string value
  Pother   = From-Workspace, To-Workspace
  Pok      = False will cancel

"changeWorkspace"
  Pcontext = Desktop handle string value
  Pother   = Workspace ID

"changeTask"
  Pother   = Task Number

"newTaskRow"
  Pcontext = RowObject buffer handle string value
  Pother   = Workspace ID

"roundtableStartup"
  Pcontext = Tabletop procedure handle string value
  Pother   = Tabletop window handle string value

"roundtableShutdown"

"sessionStartup"
  Pcontext = Session ID string value
  Pok      = False will cancel

"sessionShutdown"
*/




/* ********************  Preprocessor Definitions  ******************** */

&Scoped-define PROCEDURE-TYPE Procedure
&Scoped-define DB-AWARE no






/* *********************** Procedure Settings ************************ */



/* *************************  Create Window  ************************** */

/* DESIGN Window definition (used by the UIB)
  CREATE WINDOW Procedure ASSIGN
         HEIGHT             = 15.52
         WIDTH              = 49.4.
/* END WINDOW DEFINITION */
                                                                        */






/* ***************************  Main Block  *************************** */

SUBSCRIBE TO "evRtbUserEvent":U ANYWHERE.

/*
 Uncomment to search for legacy event handler and use if exists.
 OR, set to TRUE if to use legacy rtb_evnt.p customizations
*/

/*Muse-legacy = SEARCH("rtb_evnt.p":U) <> ?.*/



/* **********************  Internal Procedures  *********************** */

PROCEDURE evRtbUserEvent :
    /*------------------------------------------------------------------------------
      Purpose:     Roundtable Event Handler
      Parameters:  Pevent
                   Pcontext
                   Pother
                   Pok
      Notes:       Common procedure for all events.
                   RETURN a value from this procedure to return an error message
                   to the caller.
    ------------------------------------------------------------------------------*/
    DEFINE INPUT  PARAMETER Pevent   AS CHARACTER.
    DEFINE INPUT  PARAMETER Pcontext AS CHARACTER.
    DEFINE INPUT  PARAMETER Pother   AS CHARACTER.
    DEFINE OUTPUT PARAMETER Pok      AS LOGICAL INIT YES.

    DEFINE VARIABLE cError AS CHARACTER NO-UNDO INITIAL "".

    IF Muse-legacy THEN
    DO:
        /*
         Event specific processing for legacy code
        */
        RUN rtb_legacy_events (INPUT Pevent,
            INPUT Pcontext,
            INPUT Pother,
            OUTPUT Pok).
    END.
    ELSE
    DO:
    /*
    MESSAGE "Event:   " Pevent SKIP
            "Context: " Pcontext SKIP
            "Other:   " Pother
      VIEW-AS ALERT-BOX
      BUTTONS OK-CANCEL
      TITLE "Roundtable Event"
      UPDATE Pok.
    */
    END.

    RETURN cError.

END PROCEDURE.
