/*********************************************************************
 Copyright (C) 2012-2015 by appSolutions a/s ("AS") -
 www.app-solutions.com and other contributors as listed below.
 All Rights Reserved.

Permission is hereby granted, free of charge, to any person
obtaining a copy of this software and associated documentation
files (the "Software"), to deal in the Software without
restriction except as noted below, including without limitation
the rights to use,copy, modify, merge, publish, distribute,
and/or sublicense, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

 The Software and/or source code cannot be copied in whole and
 sold without meaningful modification for a profit.

 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.

 Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.

 Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in
 the documentation and/or other materials provided with
 the distribution.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
OTHER DEALINGS IN THE SOFTWARE.
*********************************************************************/
/*------------------------------------------------------------------------
   File        : scm/custom/rtb/Utils/RTBProxyHelper.cls
   Purpose     : provide OOABL access to RTB proxies.
   Syntax      :
                 DEFINE VARIABLE oRtbProxyHelper AS scm.custom.rtb.Utils.RtbProxyHelper NO-UNDO.

                 oRTBProxyHelper = NEW scm.custom.rtb.Utils.RTBProxyHelper().
                 oRTBProxyHelper:init(hAppServerHandle).

                 OR

                 oRTBProxyHelper = NEW scm.custom.rtb.Utils.RTBProxyHelper(hAppServerHandle).

   Description :
   Created     : Fri Nov 22 13:51:20 CET 2013
   Notes       : This class has been created as a wrapper around the standard RTB proxy procedures.
                 It can be used as an extension to the standard API in Roundtable.

                 NB: The code contained in this code is not part of the official RTB codebase.
                     Signatures to the proxies can change without warning when the core RTB code is udpated.
                Methods with names starting with "custRtb" are custom methods that do not map directly to standard
                RTB proxy calls. They may use combinations of proxy calls and/or other methods in this class.

                August 2014 Thomas Hansen
                Added numerous updates with cust* methods to better support version notes creation, validating task programmers list and more.
 ----------------------------------------------------------------------*/

USING Progress.Lang.*.

ROUTINE-LEVEL ON ERROR UNDO, THROW.

CLASS scm.custom.rtb.Utils.RTBProxyHelper:

    DEFINE PUBLIC PROPERTY AppServerHandle AS HANDLE NO-UNDO
        GET.
        SET.

    DEFINE PUBLIC PROPERTY DebugOn AS LOGICAL INITIAL FALSE NO-UNDO
        GET.
        SET.
    DEFINE PUBLIC PROPERTY oTempTableHelper AS scm.custom.rtb.Utils.TempTableHelper NO-UNDO
        GET.
        SET.

    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/

    CONSTRUCTOR PUBLIC RTBProxyHelper (  ):
        SUPER ().

        THIS-OBJECT:init(SESSION:HANDLE).
        oTempTableHelper = NEW scm.custom.rtb.Utils.TempTableHelper().

    END CONSTRUCTOR.

    CONSTRUCTOR PUBLIC RTBProxyHelper (
        INPUT hAppServerHandle AS HANDLE ):

        SUPER ().

        THIS-OBJECT:init(hAppServerHandle).

        ASSIGN
            AppServerHandle = hAppServerHandle.

        oTempTableHelper = NEW scm.custom.rtb.Utils.TempTableHelper().
    END CONSTRUCTOR.

    /*------------------------------------------------------------------------------
            Purpose: Check for the existence of proxy procedure
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID checkProxyProcedure(
        INPUT pcProxyProcedure AS CHARACTER,
        INPUT phPartition      AS HANDLE  ):

        IF (phPartition = SESSION:HANDLE OR phPartition = ? )AND
            SEARCH(pcProxyProcedure)  = ? THEN
            UNDO, THROW NEW Progress.Lang.AppError(SUBSTITUTE("   *** Cannot find proxy procedure : '&1'", pcProxyProcedure), 0).

        RETURN.

    END METHOD.

    /*------------------------------------------------------------------------------
               Purpose: Check for the existence of proxy procedure
               Notes:
       ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID checkProxyProcedure(
        INPUT pcProxyProcedure AS CHARACTER):

        checkProxyProcedure(pcProxyProcedure, AppServerHandle).

        RETURN.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LOGICAL custFindPriWorkspaceSrcPmod(
        INPUT pcWorkspace AS CHARACTER,
        INPUT pcPmod      AS CHARACTER  ):

        DEFINE VARIABLE lcJsonData      AS LONGCHAR  NO-UNDO.
        DEFINE VARIABLE hWsPmodData     AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hWsPmodBuf      AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hBuf            AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hQry            AS HANDLE    NO-UNDO.
        DEFINE VARIABLE lPriSrPmodFound AS LOGICAL   NO-UNDO INITIAL FALSE.
        DEFINE VARIABLE cPmodModule     AS CHARACTER NO-UNDO.

        IF pcPmod NE "":U THEN
        DO:
            lcJsonData = rtbGetWorkspaceSourcePmods(
                INPUT pcWorkspace, /* pcWorkspace */
                INPUT pcWorkspace, /* pcSrcWorkspace */
                INPUT "":U, /* pcProduct - optional  */
                INPUT custRtbGetPmodModule(pcPmod), /* pcModule */
                INPUT FALSE,
                INPUT FALSE).

            IF lcJsonData NE "":U THEN
                hWsPmodData = oTempTableHelper:ConvertJsonToTtHandle(lcJsonData).

            IF VALID-HANDLE(hWsPmodData) THEN
            DO:
                hWsPmodBuf = hWsPmodData:DEFAULT-BUFFER-HANDLE.
                CREATE QUERY hQry.
                hQry:SET-BUFFERS(hWsPmodBuf).
                hQry:QUERY-PREPARE(SUBSTITUTE("FOR EACH &1 ", hWsPmodData:NAME)).
                hQry:QUERY-OPEN().
                hQry:GET-FIRST() NO-ERROR.
                IF hWsPmodBuf:AVAILABLE THEN
                    ASSIGN
                        lPriSrPmodFound = TRUE.

                WsPmodBlock:
                REPEAT WHILE NOT hQry:QUERY-OFF-END:
                    IF hWsPmodBuf::src-status = "PRI":U THEN
                    DO:
                        ASSIGN
                            lPriSrPmodFound = TRUE.
                        LEAVE WsPmodBlock.
                    END.

                    hQry:GET-NEXT.
                END.
            END.
        END.

        RETURN lPriSrPmodFound.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hWsPmodData NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC HANDLE custGetRtbTaskDataHandleFromNumber(
        INPUT piTaskNumber AS INTEGER  ):

        DEFINE VARIABLE lcObjectJsonData AS LONGCHAR NO-UNDO.
        DEFINE VARIABLE hTaskData        AS HANDLE   NO-UNDO.

        IF piTaskNumber NE 0 THEN
        DO:
            lcObjectJsonData = rtbGetTask(piTaskNumber).
            IF lcObjectJsonData NE "":U THEN
                hTaskData = oTempTableHelper:ConvertJsonToTtHandle(lcObjectJsonData).
        END.

        RETURN hTaskData.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC HANDLE custGetRtbVerDataHandle(
        INPUT pcObjectType AS CHARACTER,
        INPUT pcObject     AS CHARACTER,
        INPUT pcPmod       AS CHARACTER,
        INPUT piVersion    AS INTEGER):

        DEFINE VARIABLE lcJsonData   AS LONGCHAR NO-UNDO.
        DEFINE VARIABLE hVersionData AS HANDLE   NO-UNDO.

        lcJsonData = rtbGetVersion(
            INPUT pcObjectType,
            INPUT pcObject,
            INPUT pcPmod,
            INPUT piVersion).

        IF lcJsonData NE "":U THEN
            hVersionData = oTempTableHelper:ConvertJsonToTtHandle(lcJsonData).

        RETURN hVersionData.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Custom method to get workspace module dir from workspace module
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC CHARACTER custRtbGetModuleDirectory(
        INPUT pcModule AS CHARACTER  ):

        DEFINE VARIABLE hData      AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hBuf       AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cModuleDir AS CHARACTER NO-UNDO.

        IF pcModule NE "":U THEN
        DO:
            hData = oTempTableHelper:ConvertJsonToTtHandle(rtbGetModuleDef(pcModule)).

            IF VALID-HANDLE(hData) THEN
            DO:
                hBuf = hData:DEFAULT-BUFFER-HANDLE.
                hBuf:FIND-FIRST ().
                ASSIGN
                    cModuleDir = hBuf::directory.
            END.
        END.

        RETURN cModuleDir.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hData NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Custom method to get the workspace module for a given pmod
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC CHARACTER custRtbGetPmodModule(
        INPUT pcPmod AS CHARACTER ):

        DEFINE VARIABLE hData       AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hBuf        AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cModuleName AS CHARACTER NO-UNDO.

        IF pcPmod NE "":U THEN
        DO:
            hData = rtbGetPmodHandle(pcPmod).

            IF VALID-HANDLE(hData) THEN
            DO:
                hBuf = hData:DEFAULT-BUFFER-HANDLE.
                hBuf:FIND-FIRST ().
                ASSIGN
                    cModuleName = hBuf::module.
            END.
        END.

        RETURN cModuleName.

        FINALLY:
            DELETE OBJECT hData NO-ERROR.
        END FINALLY.
    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Get the schema update log file for a given workspace
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR custRtbGetSchemaUpdateLog(
        INPUT pcWorkspace AS CHARACTER  ):

        DEFINE VARIABLE cWorkspaceRoot           AS CHARACTER NO-UNDO.
        DEFINE VARIABLE lcSchemaUpdateLog        AS LONGCHAR  NO-UNDO.
        DEFINE VARIABLE cSchemaUpdateLogFilename AS CHARACTER NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            cWorkspaceRoot = custGetWorkspaceRoot(pcWorkspace).
            IF cWorkspaceRoot NE "":U THEN
            DO:
                ASSIGN
                    cSchemaUpdateLogFilename = SUBSTITUTE("&1/schupd/schupd.log", cWorkspaceRoot)  .

                FILE-INFO:FILE-NAME =  cSchemaUpdateLogFilename.
                IF FILE-INFO:FULL-PATHNAME NE ? THEN
                    COPY-LOB FROM FILE cSchemaUpdateLogFilename TO lcSchemaUpdateLog.
            END.
        END.

        RETURN lcSchemaUpdateLog.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Custom method to return a list of workspaces
     Notes:   Workspaces that are hidden, archived or partner workspaces will not be included in the list
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC CHARACTER custRtbGetWorkspaceList(
        INPUT pcWorkspaceFilter     AS CHARACTER,
        INPUT plInlcudeHidden       AS LOGICAL,
        INPUT plInlcudeArchived     AS LOGICAL,
        INPUT plInlcudePartnerSites AS LOGICAL):

        DEFINE VARIABLE lcJsonData     AS LONGCHAR  NO-UNDO.
        DEFINE VARIABLE hData          AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hBuf           AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hQry           AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cWorkspaceList AS CHARACTER NO-UNDO.

        IF pcWorkspaceFilter = "":U THEN
            ASSIGN
                pcWorkspaceFilter = "*":U.

        lcJsonData = rtbGetWorkspaces(plInlcudeHidden).

        IF lcJsonData NE "":U THEN
            hData = oTempTableHelper:ConvertJsonToTtHandle(lcJsonData).

        IF VALID-HANDLE(hData) THEN
        DO:
            hBuf = hData:DEFAULT-BUFFER-HANDLE.
            CREATE QUERY hQry.
            hQry:SET-BUFFERS(hBuf).
            hQry:QUERY-PREPARE("FOR EACH " + hData:NAME).
            hQry:QUERY-OPEN().
            hQry:GET-FIRST().

            REPEAT WHILE NOT hQry:QUERY-OFF-END:
                IF NOT plInlcudeHidden AND
                    hBuf::HIDDEN THEN
                DO:
                    hQry:GET-NEXT ().
                    NEXT.
                END.

                IF NOT plInlcudeArchived AND
                    hBuf::wsArchived THEN
                DO:
                    hQry:GET-NEXT ().
                    NEXT.
                END.

                IF NOT plInlcudePartnerSites AND
                    hBuf::wsPartnerSite THEN
                DO:
                    hQry:GET-NEXT ().
                    NEXT.
                END.

                ASSIGN
                    cWorkspaceList = cWorkspaceList + ",":U + hBuf::wspace-id.
                hQry:GET-NEXT ().
            END.
        END.

        ASSIGN
            cWorkspaceList = TRIM(cWorkspaceList, ",":U).

        RETURN cWorkspaceList.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hQry NO-ERROR.
            DELETE OBJECT hData NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Custom method to return a list of workspaces with default settings
     Notes:   Workspaces that are hidden, archived or partner workspaces will not be included in the list
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC CHARACTER custRtbGetWorkspaceList(
        INPUT pcWorkspaceFilter     AS CHARACTER):

        DEFINE VARIABLE cWorkspaceList       AS CHARACTER NO-UNDO.
        DEFINE VARIABLE lIncludeHidden       AS LOGICAL   NO-UNDO.
        DEFINE VARIABLE lIncludeArchived     AS LOGICAL   NO-UNDO.
        DEFINE VARIABLE lInlcudePartnerSites AS LOGICAL   NO-UNDO.

        IF pcWorkspaceFilter = "":U THEN
            ASSIGN
                pcWorkspaceFilter = "*":U.

        cWorkspaceList = custRtbGetWorkspaceList(
            INPUT pcWorkspaceFilter,
            INPUT lIncludeHidden,
            INPUT lIncludeArchived,
            INPUT lInlcudePartnerSites).

        RETURN cWorkspaceList.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Custom method to set version notes for an veriosn record using the rowid context of an rtb_object record
     Notes:
         @param pcContext - a string with the ROWID for the rtb_object
         @param pcUpdNotes - the verison notes to be added to the rtb_version record
         @param plAppendVersionNotes - Should the version notes be appended to existing notes?
         @param pcversionNotesMatch - a string to do a MATCHES on existing version notes to see if they exist already
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID custRtbSetVerUpdNotes(
        INPUT  pcContext            AS CHARACTER,
        INPUT  pcUpdNotes           AS CHARACTER,
        INPUT  plAppendVersionNotes AS LOGICAL,
        INPUT  pcVersionNotesMatch  AS CHARACTER):

        DEFINE VARIABLE cButton     AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cErrorText  AS CHARACTER NO-UNDO.
        DEFINE VARIABLE hObjectData AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hVerData    AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hVerBuf     AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hObjBuf     AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hVerBuf2    AS HANDLE    NO-UNDO.

        IF pcContext NE "":U THEN
        DO:
            hObjectData = rtbGetObjectByRowidHandle(pcContext).

            IF VALID-HANDLE(hObjectData) AND
                hObjectData:HAS-RECORDS THEN
            DO:
                hObjBuf = hObjectData:DEFAULT-BUFFER-HANDLE.
                hObjBuf:FIND-FIRST() NO-ERROR.

                IF hObjBuf:AVAILABLE THEN
                DO:
                    /* Get object version details */
                    hVerData = rtbGetVersionHandle(
                        INPUT hObjBuf::obj-type,
                        INPUT hObjBuf::object,
                        INPUT hObjBuf::pmod,
                        INPUT INTEGER(hObjBuf::VERSION)).

                    IF VALID-HANDLE(hVerData) AND
                        hVerData:HAS-RECORDS THEN
                    DO ON ERROR UNDO, THROW:
                        hVerBuf = hVerData:DEFAULT-BUFFER-HANDLE.
                        hVerBuf:FIND-FIRST() NO-ERROR.

                        /* Check if we have a match on the pcVersionNotesMatch - if we do, then we are not going to create version notes as they are already there - possible created from something like server extensions */
                        IF hVerBuf:AVAILABLE AND
                            pcVersionNotesMatch NE "":U THEN
                            IF hVerBuf::upd-notes MATCHES pcVersionNotesMatch THEN
                            DO:
                                IF DebugOn THEN
                                    /***DIAG***/
                                    MESSAGE {&line-number} PROGRAM-NAME(1) SKIP
                                        "Version notes match : " SKIP
                                        pcVersionNotesMatch SKIP
                                        hVerBuf::upd-notes
                                        VIEW-AS ALERT-BOX INFO BUTTONS OK.

                                RETURN.
                            END.

                        /*
                         Create a copy of the original record we received
                        */
                        CREATE BUFFER hVerBuf2 FOR TABLE hVerData.
                        hVerBuf2:BUFFER-CREATE.
                        hVerBuf2:BUFFER-COPY(hVerBuf).

                        IF plAppendVersionNotes THEN
                            ASSIGN
                                hVerBuf2::upd-notes = hVerBuf2::upd-notes + "~n":U + pcUpdNotes.
                        ELSE
                            ASSIGN
                                hVerBuf2::upd-notes = pcUpdNotes.

                        ASSIGN
                            hVerBuf2::rowMod = "U".

                        rtbSetRowValues (
                            INPUT "rtbVersion":U,
                            INPUT-OUTPUT TABLE-HANDLE hVerData).

                        CATCH err AS Progress.Lang.Error :
                            UNDO, THROW err.
                        END CATCH.
                    END.
                END.
            END.
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hObjectData NO-ERROR.
            DELETE OBJECT hVerData NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Custom method to set version notes for an veriosn record using the rowid context of an rtb_object record
     Notes:
         @param pcContext - a string with the ROWID for the rtb_object
         @param pcUpdNotes - the verison notes to be added to the rtb_version record
         @param plAppendVersionNotes - Should the version notes be appended to existing notes?
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID custRtbSetVerUpdNotes(
        INPUT  pcContext            AS CHARACTER,
        INPUT  pcUpdNotes           AS CHARACTER,
        INPUT  plAppendVersionNotes AS LOGICAL):

        custRtbSetVerUpdNotes(
            INPUT pcContext,
            INPUT pcUpdNotes,
            INPUT plAppendVersionNotes,
            INPUT "":U).

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.
    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Custom method to set version notes for an veriosn record using the rowid context of an rtb_object record
     Notes:
         @param pcContext - a string with the ROWID for the rtb_object
         @param pcUpdNotes - the verison notes to be added to the rtb_version record
         @param plAppendVersionNotes - Should the version notes be appended to existing notes?
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID custRtbSetVerUpdNotes(
        INPUT  pcContext            AS CHARACTER,
        INPUT  pcUpdNotes           AS CHARACTER):

        custRtbSetVerUpdNotes(
            INPUT pcContext,
            INPUT pcUpdNotes,
            INPUT FALSE,
            INPUT "":U).

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Custom method to get paths paths for a workspace
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID custGetWorkspacePaths(
        INPUT  pcWorkspace           AS CHARACTER,
        OUTPUT pcWorkspacePath       AS CHARACTER,
        OUTPUT pcWorkspaceServerPath AS CHARACTER,
        OUTPUT pcWorkspaceRootPath   AS CHARACTER  ):

        DEFINE VARIABLE hWorkspaceData AS HANDLE NO-UNDO.
        DEFINE VARIABLE hBuf           AS HANDLE NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            /* Get the workspace record */
            hWorkspaceData = oTempTableHelper:ConvertJsonToTtHandle(rtbGetWorkspace (pcWorkspace)).

            hBuf = hWorkspaceData:DEFAULT-BUFFER-HANDLE.
            hBuf:FIND-FIRST() NO-ERROR.

            /* Get the current workspace path and root path  */
            IF hBuf:AVAILABLE THEN
                ASSIGN
                    pcWorkspacePath       = hBuf::wspace-path
                    pcWorkspaceServerPath = hBuf::wspace-path-server
                    pcWorkspaceRootPath   = ENTRY(1, pcWorkspacePath).
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hWorkspaceData NO-ERROR.
        END FINALLY.
    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Custom method to get the root directory for a workspace
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC CHARACTER custGetWorkspaceRoot(
        INPUT  pcWorkspace AS CHARACTER  ):

        DEFINE VARIABLE cWorkspacePath       AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cWorkspaceServerPath AS CHARACTER NO-UNDO.
        DEFINE VARIABLE cWorkspaceRoot       AS CHARACTER NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            custGetWorkspacePaths(
                INPUT  pcWorkspace,
                OUTPUT cWorkspacePath,
                OUTPUT cWorkspaceServerPath,
                OUTPUT cWorkspaceRoot).
        END.

        RETURN cWorkspaceRoot.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Custom method to create a task and set properties on the task
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC INTEGER custRtbCreateTask(
        INPUT  pcWorkspace       AS CHARACTER,
        INPUT  pcTaskSummary     AS CHARACTER,
        INPUT  pcTaskDescription AS CHARACTER,
        INPUT  pcTaskUserTaskref AS CHARACTER,
        INPUT  pcProgrammer      AS CHARACTER,
        INPUT  pcManager         AS CHARACTER):

        DEFINE VARIABLE iTaskNum  AS INTEGER   NO-UNDO.
        DEFINE VARIABLE cRowident AS CHARACTER NO-UNDO.
        DEFINE VARIABLE hTaskData AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hBuf      AS HANDLE    NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            ASSIGN
                cRowident = "schema".

            /* Get an empty rtbTask schema */
            rtbGetRowValues (
                INPUT-OUTPUT cRowIdent,
                INPUT 0,
                INPUT "rtbTask",
                INPUT-OUTPUT TABLE-HANDLE hTaskdata,
                INPUT FALSE).

            IF VALID-HANDLE(hTaskData) THEN
            DO:
                hBuf = hTaskData:DEFAULT-BUFFER-HANDLE.
                hBuf:BUFFER-CREATE().

                ASSIGN
                    hBuf::rowMod          = "A"
                    hBuf::wspace-id       = pcWorkspace
                    hBuf::summary         = pcTaskSummary
                    hBuf::TaskDescription = pcTaskDescription
                    hBuf::user-task-ref   = pcTaskUserTaskref
                    hBuf::taskDescription = pcTaskDescription
                    hBuf::programmer      = REPLACE(pcProgrammer, ",":U, ";":U)
                    hBuf::manager         = REPLACE(pcManager, ",":U, ";":U).

                rtbSetRowValues (
                    INPUT "rtbTask",
                    INPUT-OUTPUT TABLE-HANDLE hTaskdata).

                IF VALID-HANDLE(hTaskdata) THEN
                DO:
                    /* read the task number and other committed data */
                    hBuf = hTaskdata:DEFAULT-BUFFER-HANDLE.
                    hBuf:FIND-FIRST().

                    ASSIGN
                        iTaskNum = hBuf::task-num.
                END.
            END.
        END.

        RETURN iTaskNum.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hTaskdata NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Custom method to set or update the summary on a task
     Notes:   This is needed as a separate method because the standard proxy to create tasks does not take summary as an input parameter.
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID custRtbSetTaskSummary(
        INPUT  piTaskNum     AS INTEGER,
        INPUT  pcTaskSummary AS CHARACTER):

        DEFINE VARIABLE hTaskdata AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hBuf      AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cRowIdent AS CHARACTER NO-UNDO INITIAL "FIRST".

        IF piTaskNum NE 0 THEN
        DO:
            hTaskdata = rtbGetTaskHandle(piTaskNum).

            IF VALID-HANDLE(hTaskdata) THEN
            DO:
                hBuf = hTaskdata:DEFAULT-BUFFER-HANDLE.
                hBuf:FIND-FIRST().

                ASSIGN
                    hBuf::rowMod  = "U"
                    hBuf::summary = pcTaskSummary.

                rtbSetRowValues(
                    INPUT "rtbTask",
                    INPUT-OUTPUT TABLE-HANDLE hTaskdata).
            END.
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hTaskdata NO-ERROR.
        END FINALLY.
    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Custom method to set the user-task-ref field on a task
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID custRtbSetTaskUserRef(
        INPUT  piTaskNum     AS INTEGER,
        INPUT  pcUserTaskRef AS CHARACTER):

        DEFINE VARIABLE hTaskdata  AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hBuf       AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cRowIdent  AS CHARACTER NO-UNDO INITIAL "FIRST".
        DEFINE VARIABLE lcJsonData AS LONGCHAR  NO-UNDO.

        IF piTaskNum NE 0 THEN
        DO:
            hTaskdata = rtbGetTaskHandle(piTaskNum).

            IF VALID-HANDLE(hTaskdata) THEN
            DO:
                hBuf = hTaskdata:DEFAULT-BUFFER-HANDLE.
                hBuf:FIND-FIRST().

                ASSIGN
                    hBuf::rowMod        = "U"
                    hBuf::user-task-ref = pcUserTaskRef.

                rtbSetRowValues(
                    INPUT "rtbTask",
                    INPUT-OUTPUT TABLE-HANDLE hTaskdata).
            END.
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hTaskdata NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:      Validate the rtb_task.programmer field withthe passed in list of programmers
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID custValidateRtbTaskProgrammers(
        INPUT piTaskNum            AS INTEGER,
        INPUT pcTaskProgrammerList AS CHARACTER ):

        DEFINE VARIABLE hTaskData   AS HANDLE    NO-UNDO.
        DEFINE VARIABLE hTaskBuffer AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cError      AS CHARACTER NO-UNDO.

        IF piTaskNum NE 0 THEN
        DO:
            hTaskData = rtbGetTaskHandle(piTaskNum).

            IF VALID-HANDLE(hTaskData) THEN
            DO:
                hTaskBuffer = hTaskData:DEFAULT-BUFFER-HANDLE.
                hTaskBuffer:FIND-FIRST("") NO-ERROR.
                IF hTaskBuffer::programmer NE pcTaskProgrammerList THEN
                DO:
                    ASSIGN
                        cError = SUBSTITUTE("Task programmer list for task '&1' not set correctly to : '&2'", piTaskNum, pcTaskProgrammerList).
                    UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
                END.
            END.
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hTaskData NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID init(
        INPUT hAppServerHandle AS HANDLE ):

        ASSIGN
            AppServerHandle = hAppServerHandle.

        RETURN.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC HANDLE rtbApplySchemaUpdateList(
        INPUT                 pcWorkspace AS CHARACTER,
        INPUT                 plDelta     AS LOGICAL):

        DEFINE VARIABLE cError       AS CHARACTER NO-UNDO.
        DEFINE VARIABLE hDsSchupd    AS HANDLE    NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR  NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbApplySchemaUpdateList.p":U).

            RUN rtb/proxy/p/rtbApplySchemaUpdateList.p ON AppServerHandle (
                INPUT  pcWorkspace,
                INPUT  TRUE, /* plDelta */
                OUTPUT DATASET-HANDLE hDsSchupd,
                OUTPUT cError).

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
        END.

        RETURN hDsSchupd.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR rtbApplyWorkspaceImportList(
        INPUT  pcWorkspace AS CHARACTER):

        DEFINE VARIABLE hTTImportLog AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cError       AS CHARACTER NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR  NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbApplyWorkspaceImportList.p":U).

            RUN rtb/proxy/p/rtbApplyWorkspaceImportList.p ON AppServerHandle (
                INPUT  pcWorkspace,
                OUTPUT cError,
                OUTPUT TABLE-HANDLE hTTImportLog).

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError(cError, 0).

            IF VALID-HANDLE(hTTImportLog) THEN
                lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hTTImportLog).
            ELSE
                ASSIGN
                    lcJsonResult = "":U.
        END.

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hTTImportLog NO-ERROR.
        END FINALLY.
    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC HANDLE rtbBuildDeploymentSchemaUpdateList(
        INPUT                 pcWorkspace     AS CHARACTER,
        INPUT                 pcSiteCode      AS CHARACTER,
        INPUT                 piSequence      AS INTEGER):

        DEFINE VARIABLE cError       AS CHARACTER NO-UNDO.
        DEFINE VARIABLE hDS          AS HANDLE    NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR  NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbBuildDeploymentSchemaUpdateList.p":U).

            RUN rtb/proxy/p/rtbBuildDeploymentSchemaUpdateList.p ON AppServerHandle (
                INPUT  pcWorkspace,
                INPUT  pcSiteCode ,
                INPUT  piSequence,
                OUTPUT cError,
                OUTPUT DATASET-HANDLE hDS).

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
        END.

        RETURN hDS.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR rtbBuildWorkspaceImportList(
        INPUT               pcWorkspace         AS CHARACTER,
        INPUT               pcSourceWorkspaces  AS CHARACTER,
        INPUT               pcPmods             AS CHARACTER,
        INPUT               pcTasks             AS CHARACTER,
        INPUT               piReleaseNum        AS INTEGER,
        INPUT               plHigherVerOnly     AS LOGICAL,
        INPUT               plCurrReleaseOnly   AS LOGICAL,
        INPUT               plCompleteTasksOnly AS LOGICAL):

        DEFINE VARIABLE hTTImportControl AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cError           AS CHARACTER NO-UNDO.
        DEFINE VARIABLE lcJsonResult     AS LONGCHAR  NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbBuildWorkspaceImportList.p":U).

            RUN rtb/proxy/p/rtbBuildWorkspaceImportList.p ON AppServerHandle(
                INPUT  pcWorkspace,
                INPUT  pcSourceWorkspaces,
                INPUT  pcPmods,
                INPUT  pcTasks,
                INPUT  piReleaseNum,
                INPUT  plHigherVerOnly,
                INPUT  plCurrReleaseOnly,
                INPUT  plCompleteTasksOnly,
                OUTPUT TABLE-HANDLE hTTImportControl,
                OUTPUT cError).

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError(cError, 0).

            IF VALID-HANDLE(hTTImportControl) THEN
                lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hTTImportControl).
        END.

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hTTImportControl NO-ERROR.
        END FINALLY.
    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC HANDLE rtbBuildWorkspaceSchemaUpdateList(
        INPUT  pcWorkspace AS CHARACTER,
        INPUT  pcDbase     AS CHARACTER):

        DEFINE VARIABLE hDsSchupd    AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cError       AS CHARACTER NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR  NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbBuildWorkspaceSchemaUpdateList.p":U).

            RUN rtb/proxy/p/rtbBuildWorkspaceSchemaUpdateList.p ON AppServerHandle (
                INPUT  pcWorkspace,
                INPUT  pcDbase,
                OUTPUT cError,
                OUTPUT DATASET-HANDLE hDsSchupd).

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
        END.

        RETURN hDsSchupd.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: A proxy call to check the dependencies of a task
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC CHARACTER rtbCheckTaskDependencies(
        INPUT  piTaskNum   AS INTEGER):

        DEFINE VARIABLE cError AS CHARACTER NO-UNDO.

        IF piTaskNum NE 0 THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbCheckTaskDependencies.p":U).

            RUN rtb/proxy/p/rtbCheckTaskDependencies.p ON AppServerHandle (
                INPUT  piTaskNum,
                OUTPUT cError).
        END.

        RETURN cError.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: proxy call to complete a task. Note that no checks are made - the task is just completed.
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID rtbCompleteTask(
        INPUT  piTaskNum   AS INTEGER):

        DEFINE VARIABLE cError AS CHARACTER NO-UNDO.

        IF piTaskNum NE 0 THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbCompleteTask.p":U).

            RUN rtb/proxy/p/rtbCompleteTask.p ON AppServerHandle (
                INPUT  piTaskNum,
                OUTPUT cError).

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID rtbCreateRelease(
        INPUT  pcWspaceId    AS CHARACTER,
        INPUT  pcReleaseNote AS CHARACTER):

        DEFINE VARIABLE cError AS CHARACTER NO-UNDO.

        IF pcWspaceId NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbCreateRelease.p":U).

            RUN rtb/proxy/p/rtbCreateRelease.p ON AppServerHandle (
                INPUT pcWspaceId,
                INPUT pcReleaseNote,
                OUTPUT cError).

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose: Create a task in Roundtable using standard proxy calls.
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC INTEGER rtbCreateTask(
        INPUT  pcWorkspace   AS CHARACTER):

        DEFINE VARIABLE cError   AS CHARACTER NO-UNDO.
        DEFINE VARIABLE iTaskNum AS INTEGER   NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbCreateTask.p":U).

            RUN rtb/proxy/p/rtbCreateTask.p ON AppServerHandle (
                INPUT  pcWorkspace,
                OUTPUT iTaskNum,
                OUTPUT cError).

            IF cError NE "":U THEN
                UNDO,  THROW NEW Progress.Lang.AppError(cError, 0).

        END.

        RETURN iTaskNum.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID rtbDeleteSchemaUpdateList(
        INPUT  pcWorkspaceId AS CHARACTER,
        INPUT  pcSiteCode    AS CHARACTER):

        DEFINE VARIABLE cError AS CHARACTER NO-UNDO.

        IF pcWorkspaceId NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbDeleteSchemaUpdateList.p":U).

            RUN rtb/proxy/p/rtbDeleteSchemaUpdateList.p ON AppServerHandle (
                INPUT  pcWorkspaceId,
                INPUT  pcSiteCode,
                OUTPUT cError).

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID rtbDeleteWorkspaceImportList(
        INPUT  pcWorkspaceId AS CHARACTER):

        DEFINE VARIABLE cError AS CHARACTER NO-UNDO.

        IF pcWorkspaceId NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbDeleteWorkspaceImportList.p":U).

            RUN rtb/proxy/p/rtbDeleteWorkspaceImportList.p ON AppServerHandle (
                INPUT pcWorkspaceId,
                OUTPUT cError).

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR rtbGetDeployments(
        INPUT    pcWorkspace    AS CHARACTER,
        INPUT    pcSiteCode     AS CHARACTER):

        DEFINE VARIABLE cRowIdent     AS CHARACTER NO-UNDO INITIAL "FIRST".
        DEFINE VARIABLE iRowsToReturn AS INTEGER   NO-UNDO.
        DEFINE VARIABLE hTT           AS HANDLE    NO-UNDO.
        DEFINE VARIABLE lcJsonResult  AS LONGCHAR  NO-UNDO.

        hTT = rtbGetDeploymentsHandle(
            INPUT-OUTPUT cRowIdent,
            INPUT    iRowsToReturn,
            INPUT    pcWorkspace,
            INPUT    pcSiteCode).

        IF VALID-HANDLE(hTT) THEN
            lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hTT).

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hTT NO-ERROR.
        END FINALLY.
    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC HANDLE rtbGetDeploymentsHandle(
        INPUT-OUTPUT pcRowIdent     AS CHARACTER,
        INPUT        piRowsToReturn AS INTEGER,
        INPUT        pcWorkspace    AS CHARACTER,
        INPUT        pcSiteCode     AS CHARACTER):

        DEFINE VARIABLE hTT          AS HANDLE   NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetDeployments":U).

            RUN rtb/proxy/p/rtbGetDeployments ON AppServerHandle (
                INPUT-OUTPUT pcRowIdent,
                INPUT        piRowsToReturn,
                INPUT       pcWorkspace,
                INPUT       pcSiteCode,
                OUTPUT      TABLE-HANDLE hTT).
        END.

        IF VALID-HANDLE(hTT) THEN
            RETURN hTT.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Wrapper for Proxy procedure to return a Workspace Module definition
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR rtbGetModuleDef(
        INPUT  pcModule AS CHARACTER):

        DEFINE VARIABLE hTT          AS HANDLE   NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR NO-UNDO.

        IF pcModule NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetModuleDef.p":U).

            RUN rtb/proxy/p/rtbGetModuleDef.p ON AppServerHandle (
                INPUT  pcModule,
                OUTPUT TABLE-HANDLE hTT).

            IF VALID-HANDLE(hTT) THEN
                lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hTT).
        END.

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hTT NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Wrapper method to call proxy to get workspace module definitions.
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR rtbGetModuleDefs(
        INPUT-OUTPUT pcRowIdent     AS CHARACTER,
        INPUT        piRowsToReturn AS INTEGER):

        DEFINE VARIABLE hTT          AS HANDLE   NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR NO-UNDO.

        checkProxyProcedure("rtb/proxy/p/rtbGetModuleDefs.p":U).

        RUN rtb/proxy/p/rtbGetModuleDefs.p ON AppServerHandle (
            INPUT-OUTPUT pcRowIdent,
            INPUT        piRowsToReturn,
            OUTPUT       TABLE-HANDLE hTT).

        IF VALID-HANDLE(hTT) THEN
            lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hTT).

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hTT NO-ERROR.
        END FINALLY.
    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR rtbGetObjectByRowid(
        INPUT pcContext AS CHARACTER):

        DEFINE VARIABLE hObjectData  AS HANDLE   NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR NO-UNDO.

        IF pcContext NE "":U THEN
        DO:
            hObjectData = rtbGetObjectByRowidHandle(INPUT  pcContext).

            IF VALID-HANDLE(hObjectData) THEN
                lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hObjectData).
        END.

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hObjectData NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC HANDLE rtbGetObjectByRowidHandle(
        INPUT pcContext AS CHARACTER):

        DEFINE VARIABLE hObjectData AS HANDLE NO-UNDO.

        IF pcContext NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetObjectByRowid.p":U).

            RUN rtb/proxy/p/rtbGetObjectByRowid ON AppServerHandle (
                INPUT  pcContext,
                OUTPUT TABLE-HANDLE hObjectData).
        END.

        IF VALID-HANDLE(hObjectData) THEN
            RETURN hObjectData.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID rtbGetObjectParts(
        INPUT  pcRowident  AS CHARACTER,
        OUTPUT pcPartslist AS CHARACTER):

        IF pcRowident NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetObjectParts.p":U).

            RUN rtb/proxy/p/rtbGetObjectParts.p ON AppServerHandle(
                INPUT  pcRowident,
                OUTPUT pcPartslist).
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID rtbGetObjectPath(
        INPUT  pcRowident AS CHARACTE,
        INPUT  piPartNum  AS INTEGER,
        OUTPUT pcPathname AS CHARACTER):

        IF pcRowident NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetObjectPath.p":U).

            RUN rtb/proxy/p/rtbGetObjectPath.p ON AppServerHandle(
                INPUT pcRowident,
                INPUT piPartNum,
                OUTPUT pcPathname).
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.
    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR rtbGetPmod(
        INPUT  pcPmod AS CHARACTER):

        DEFINE VARIABLE hTT          AS HANDLE   NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR NO-UNDO.

        IF pcPmod NE "":U THEN
        DO:
            IF pcPmod NE "":U THEN
            DO:
                hTT = rtbGetPmodHandle(pcPmod).

                IF VALID-HANDLE(hTT) THEN
                    lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hTT).
            END.
        END.

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hTT NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC HANDLE rtbGetPmodHandle(
        INPUT  pcPmod AS CHARACTER  ):

        DEFINE VARIABLE hTT AS HANDLE NO-UNDO.

        IF pcPmod NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetPmod.p":U).

            RUN rtb/proxy/p/rtbGetPmod.p ON AppServerHandle (
                INPUT  pcPmod,
                OUTPUT TABLE-HANDLE hTT).
        END.

        IF VALID-HANDLE(hTT) THEN
            RETURN hTT.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.
    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Get a rtb_release record from the RTB proxy and convert this to JSON which is returned to the caller.
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR rtbGetRelease(
        INPUT-OUTPUT pcRowIdent     AS CHARACTER,
        INPUT        piRowsToReturn AS INTEGER,
        INPUT        pcWorkspace    AS CHARACTER,
        INPUT        piRelease      AS INTEGER):

        DEFINE VARIABLE hTT          AS HANDLE   NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            hTT = rtbGetReleaseHandle(
                INPUT-OUTPUT pcRowIdent,
                INPUT        piRowsToReturn,
                INPUT        pcWorkspace,
                INPUT        piRelease).

            IF VALID-HANDLE(hTT) THEN
                lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hTT).
        END.

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hTT NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:  Get a rtb_release temp-table handle  from the RTB proxy and return this to the caller.
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC HANDLE rtbGetReleaseHandle(
        INPUT-OUTPUT pcRowIdent     AS CHARACTER,
        INPUT        piRowsToReturn AS INTEGER,
        INPUT        pcWorkspace    AS CHARACTER,
        INPUT        piRelease      AS INTEGER):

        DEFINE VARIABLE hTT AS HANDLE NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetRelease.p":U).

            RUN rtb/proxy/p/rtbGetRelease.p ON AppServerHandle (
                INPUT-OUTPUT pcRowIdent,
                INPUT        piRowsToReturn,
                INPUT        pcWorkspace,
                INPUT        piRelease,
                OUTPUT       TABLE-HANDLE hTT).

            IF VALID-HANDLE(hTT) THEN
                RETURN hTT.
        END.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: method to call Proxy procedure to get RowObject data from RTB repository
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID rtbGetRowValues (
        INPUT-OUTPUT              pcRowident     AS CHARACTER,
        INPUT                     piRowsToReturn AS INTEGER,
        INPUT                     pcResultSet    AS CHARACTER,
        INPUT-OUTPUT TABLE-HANDLE phTT,
        INPUT                     plSearch       AS LOGICAL):

        IF pcResultSet NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetRowValues.p":U).

            RUN rtb/proxy/p/rtbGetRowValues.p ON AppServerHandle (
                INPUT-OUTPUT pcRowident,
                INPUT        piRowsToReturn,
                INPUT        pcResultSet,
                INPUT-OUTPUT TABLE-HANDLE phTT,
                INPUT        plSearch).
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT phTT NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC HANDLE rtbGetSchemaUpdateList (
        INPUT  pcWorkspace  AS CHARACTER,
        INPUT  pcSiteCode   AS CHARACTER):

        DEFINE VARIABLE cError       AS CHARACTER NO-UNDO.
        DEFINE VARIABLE hDsSchupd    AS HANDLE    NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR  NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetSchemaUpdateList.p":U).

            RUN rtb/proxy/p/rtbGetSchemaUpdateList.p ON AppServerHandle (
                INPUT pcWorkspace,
                INPUT pcSiteCode,
                OUTPUT DATASET-HANDLE hDsSchupd,
                OUTPUT cError).

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
        END.

        RETURN hDsSchupd.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC CHARACTER rtbGetSessionProperty(
        INPUT  pcProperty AS CHARACTER):

        DEFINE VARIABLE cValue AS CHARACTER NO-UNDO.

        IF pcProperty NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetSessionProperty.p":U).

            RUN rtb/proxy/p/rtbGetSessionProperty.p ON AppServerHandle (
                INPUT  pcProperty,
                OUTPUT cValue).
        END.

        RETURN cValue.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:    Get a task using the standard RTB proxy.
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR rtbGetTask(
        INPUT  piTaskNumber AS INTEGER):

        DEFINE VARIABLE hTT          AS HANDLE   NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR NO-UNDO.

        IF piTaskNumber NE 0 THEN
        DO:
            hTT = rtbGetTaskHandle(INPUT piTaskNumber).

            IF VALID-HANDLE(hTT) THEN
                lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hTT).
        END.

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC HANDLE rtbGetTaskHandle(
        INPUT  piTaskNumber AS INTEGER):

        DEFINE VARIABLE hTT          AS HANDLE   NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR NO-UNDO.

        IF piTaskNumber NE 0 THEN
        DO:
            IF piTaskNumber NE 0 THEN
            DO:
                checkProxyProcedure("rtb/proxy/p/rtbGetTask.p":U).

                RUN rtb/proxy/p/rtbGetTask.p ON AppServerHandle (
                    INPUT piTaskNumber,
                    OUTPUT TABLE-HANDLE hTT).
            END.

            IF VALID-HANDLE(hTT) THEN
                RETURN hTT.
        END.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Get tasks from Roundtable using the standard RTB proxy API
     Notes:

    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR rtbGetTasks(
        INPUT  pcWorkspace    AS CHARACTER,
        INPUT  pcUser         AS CHARACTER,
        INPUT  plCompleted    AS LOGICAL):

        DEFINE VARIABLE hTT          AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cRowident    AS CHARACTER NO-UNDO INITIAL "FIRST".
        DEFINE VARIABLE lcJsonResult AS LONGCHAR  NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            IF pcWorkspace NE "":U THEN
            DO:
                hTT = rtbGetTasksHandle(
                    INPUT-OUTPUT cRowident,
                    INPUT        0,
                    INPUT        pcWorkspace,
                    INPUT        pcUser,
                    INPUT        plCompleted).

                IF VALID-HANDLE(hTT) THEN
                    lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hTT).
            END.
        END.

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hTT NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC HANDLE rtbGetTasksHandle(
        INPUT-OUTPUT pcRowIdent    AS CHARACTER,
        INPUT        piRowsToBatch AS INTEGER,
        INPUT        pcWorkspace   AS CHARACTER,
        INPUT        pcUser        AS CHARACTER,
        INPUT        plCompleted   AS LOGICAL):

        DEFINE VARIABLE hTT          AS HANDLE   NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR NO-UNDO.

        checkProxyProcedure("rtb/proxy/p/rtbGetTasks.p":U).

        IF pcUser= "*":U THEN
            ASSIGN
                pcUser = "":U.

        RUN rtb/proxy/p/rtbGetTasks.p ON AppServerHandle (
            INPUT-OUTPUT pcRowIdent,
            INPUT               piRowsToBatch,
            INPUT               pcWorkspace,
            INPUT               pcUser,
            INPUT               plCompleted,
            OUTPUT TABLE-HANDLE hTT).

        IF VALID-HANDLE(hTT) THEN
            RETURN hTT.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:

        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR rtbGetVersion(
        INPUT  pcObjType AS CHARACTER,
        INPUT  pcObject  AS CHARACTER,
        INPUT  pcPmod    AS CHARACTER,
        INPUT  piVersion AS INTEGER):

        DEFINE VARIABLE hTT          AS HANDLE   NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR NO-UNDO.

        IF pcObjType NE "":U AND
            pcObject NE "":U AND
            pcPmod NE "":U AND
            piVersion NE 0 THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetVersion.p":U).

            RUN rtb/proxy/p/rtbGetVersion.p ON AppServerHandle (
                INPUT pcObjType,
                INPUT pcObject,
                INPUT pcPmod,
                INPUT piVersion,
                OUTPUT TABLE-HANDLE hTT).

            IF VALID-HANDLE(hTT) THEN
                lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hTT).
        END.
        ELSE
            UNDO, THROW NEW AppError("Invalid parameters passed in to get version record", 0).

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hTT NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR rtbGetVersionByRowid(
        INPUT               pcRowid AS CHARACTER):

        DEFINE VARIABLE hVersionData AS HANDLE   NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR NO-UNDO.

        IF pcRowid NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetVersionByRowid.p":U).

            RUN rtb/proxy/p/rtbGetVersionByRowid.p ON AppServerHandle(
                INPUT pcRowid,
                OUTPUT TABLE-HANDLE hVersionData).

            IF VALID-HANDLE(hVersionData) THEN
                lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hVersionData).
        END.
        ELSE
            UNDO, THROW NEW AppError(SUBSTITUTE("Invalid VersionRowid passed in : &1", pcRowid), 0).

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hVersionData NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC HANDLE rtbgetVersionHandle(
        INPUT  pcObjType AS CHARACTER,
        INPUT  pcObject  AS CHARACTER,
        INPUT  pcPmod    AS CHARACTER,
        INPUT  piVersion AS INTEGER):

        DEFINE VARIABLE hTT AS HANDLE NO-UNDO.

        IF pcObjType NE "":U AND
            pcObject NE "":U AND
            pcPmod NE "":U AND
            piVersion NE 0 THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetVersion.p":U).

            RUN rtb/proxy/p/rtbGetVersion.p ON AppServerHandle (
                INPUT pcObjType,
                INPUT pcObject,
                INPUT pcPmod,
                INPUT piVersion,
                OUTPUT TABLE-HANDLE hTT).

            RETURN hTT.
        END.
        ELSE
            UNDO, THROW NEW AppError("Invalid parameters passed in to get version record", 0).

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:

        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose: Call the RTB Proxy procedure rtb/proxy/p/rtbGetWorkspace.p and
                     return the temp-table handle with the search results.
            Notes:
            @param - pcWorkspaceId - the ID of the workspace to validate
            @param - phTT - TABLE-HANDLE with the temp-table data for the workspace table
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC HANDLE rtbGetWorkspaceHandle(
        INPUT  pcWorkspaceId AS CHARACTER):

        DEFINE VARIABLE hTT          AS HANDLE   NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR NO-UNDO.

        IF pcWorkspaceId NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetWorkspace.p":U).

            RUN rtb/proxy/p/rtbGetWorkspace.p ON AppServerHandle (
                INPUT               pcWorkspaceId,
                OUTPUT TABLE-HANDLE hTT).
        END.

        RETURN hTT.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC HANDLE rtbGetWorkspaceImportsHandle(
        INPUT pcWorkspace AS CHARACTER):

        DEFINE VARIABLE hImportsData AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cRowIdent    AS CHARACTER NO-UNDO INITIAL "FIRST".

        IF pcWorkspace NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetWorkspaceImports.p":U).

            RUN rtb/proxy/p/rtbGetWorkspaceImports.p ON AppServerHandle (
                INPUT-OUTPUT cRowIdent,
                INPUT 0,
                INPUT pcWorkspace,
                OUTPUT TABLE-HANDLE hImportsData).
        END.

        RETURN hImportsData.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Wrapper method for proxy procedure to return modules for a given workspace
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR rtbGetWorkspaceModules(
        INPUT-OUTPUT pcRowIdent     AS CHARACTER,
        INPUT        piRowsToReturn AS INTEGER,
        INPUT        pcWorkspace    AS CHARACTER):

        DEFINE VARIABLE hTT          AS HANDLE   NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetWorkspaceModules.p":U).

            RUN rtb/proxy/p/rtbGetWorkspaceModules.p ON AppServerHandle (
                INPUT-OUTPUT pcRowIdent,
                INPUT        piRowsToReturn,
                INPUT        pcWorkspace,
                OUTPUT       TABLE-HANDLE hTT).

            IF VALID-HANDLE(hTT) THEN
                lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hTT).
        END.

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hTT NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose:  Proxy method wrapper to return non-hidden Roundtable workspaces.
     Notes:
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC LONGCHAR rtbGetWorkspaces(
        INPUT        plHidden       AS LOGICAL /* include hidden */):

        DEFINE VARIABLE cRowIdent     AS CHARACTER NO-UNDO.
        DEFINE VARIABLE iRowsToreturn AS INTEGER   NO-UNDO.
        DEFINE VARIABLE hTT           AS HANDLE    NO-UNDO.
        DEFINE VARIABLE lcJsonResult  AS LONGCHAR  NO-UNDO.

        checkProxyProcedure("rtb/proxy/p/rtbGetWorkspaces.p":U).

        RUN rtb/proxy/p/rtbGetWorkspaces.p ON AppServerHandle (
            INPUT-OUTPUT cRowIdent,
            INPUT        iRowsToReturn,
            INPUT        plHidden, /* include hidden */
            OUTPUT       TABLE-HANDLE hTT).

        IF VALID-HANDLE(hTT) THEN
            lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hTT).

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hTT NO-ERROR.
        END FINALLY.
    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose: Check if a workspace with a given name exists in the repository
            Notes:
            @param - pcWorkspaceId - the ID of the workspace to validate
        -----------------------------------------------------------*/

    METHOD PUBLIC LOGICAL rtbWorkspaceExists(
        INPUT  pcWorkspaceId AS CHARACTER):

        DEFINE VARIABLE hWorkspaceData   AS HANDLE   NO-UNDO.
        DEFINE VARIABLE hBuf             AS HANDLE   NO-UNDO.
        DEFINE VARIABLE lcJsonData       AS LONGCHAR NO-UNDO.
        DEFINE VARIABLE lWorkspaceExists AS LOGICAL  NO-UNDO INITIAL FALSE.

        IF pcWorkspaceId NE "":U THEN
        DO:
            /* Get the workspace record */
            lcJsonData = rtbGetWorkspace (pcWorkspaceId).

            IF lcJsonData NE ? THEN
                hWorkspaceData = oTempTableHelper:ConvertJsonToTtHandle(lcJsonData).

            IF VALID-HANDLE(hWorkspaceData) THEN
            DO:
                hBuf = hWorkspaceData:DEFAULT-BUFFER-HANDLE.
                hBuf:FIND-FIRST(SUBSTITUTE(" where wspace-id = &1", QUOTER(pcWorkspaceId))).

                IF hBuf:AVAILABLE THEN
                    lWorkspaceExists = TRUE.
                ELSE
                    lWorkspaceExists = FALSE.
            END.
        END.

        RETURN lWorkspaceExists.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hWorkspaceData NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose: Call the RTB Proxy procedure rtb/proxy/p/rtbGetWorkspace.p and
                     return the temp-table handle with the search results.
            Notes:
            @param - pcWorkspaceId - the ID of the workspace to validate
            @param - phTT - TABLE-HANDLE with the temp-table data for the workspace table
    ------------------------------------------------------------------------------*/
    METHOD PUBLIC LONGCHAR rtbGetWorkspace(
        INPUT  pcWorkspaceId AS CHARACTER):

        DEFINE VARIABLE hTT          AS HANDLE   NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR NO-UNDO.

        IF pcWorkspaceId NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetWorkspace.p":U).

            RUN rtb/proxy/p/rtbGetWorkspace.p ON AppServerHandle (
                INPUT               pcWorkspaceId,
                OUTPUT TABLE-HANDLE hTT).

            IF VALID-HANDLE(hTT) THEN
                lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hTT).
        END.

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.
    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
            @param - pcWorkspaceId - the ID of the workspace to validate
            @return - LONGCHAR with JSON data for the workspace import control table
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR rtbGetWorkspaceImports(
        INPUT pcWorkspace AS CHARACTER):

        DEFINE VARIABLE hImportsData AS HANDLE    NO-UNDO.
        DEFINE VARIABLE cRowIdent    AS CHARACTER NO-UNDO INITIAL "FIRST".
        DEFINE VARIABLE lcJsonResult AS LONGCHAR  NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetWorkspaceImports.p":U).

            RUN rtb/proxy/p/rtbGetWorkspaceImports.p ON AppServerHandle (
                INPUT-OUTPUT cRowIdent,
                INPUT 0,
                INPUT pcWorkspace,
                OUTPUT TABLE-HANDLE hImportsData).

            IF VALID-HANDLE(hImportsData) AND
                hImportsData:HAS-RECORDS THEN
                lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hImportsData).
        END.

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hImportsData NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR rtbGetWorkspaceObjects(
        INPUT  piRowsToReturn AS INTEGER,
        INPUT  pcWorkspaceId  AS CHARACTER,
        INPUT  pcObjType      AS CHARACTER,
        INPUT  pcObjectFilter AS CHARACTER):

        DEFINE VARIABLE cRowIdent             AS CHARACTER NO-UNDO INITIAL "FIRST".
        DEFINE VARIABLE hWorkspaceObjectsData AS HANDLE    NO-UNDO.
        DEFINE VARIABLE lcJsonResult          AS LONGCHAR  NO-UNDO.

        IF pcWorkspaceId NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetWorkspaceObjects.p":U).

            RUN rtb/proxy/p/rtbGetWorkspaceObjects.p  ON AppServerHandle (
                INPUT-OUTPUT cRowIdent,
                INPUT piRowsToReturn,
                INPUT pcWorkspaceId,
                INPUT pcObjType,
                INPUT pcObjectFilter,
                OUTPUT TABLE-HANDLE hWorkspaceObjectsData).

            IF VALID-HANDLE(hWorkspaceObjectsData) THEN
                lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hWorkspaceObjectsData).
        END.

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hWorkspaceObjectsData NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR rtbGetWorkspaceReleases(
        INPUT               pcWorkspaceId    AS CHARACTER):

        DEFINE VARIABLE cRowIdent    AS CHARACTER NO-UNDO INITIAL "FIRST".
        DEFINE VARIABLE hReleaseData AS HANDLE    NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR  NO-UNDO.

        IF pcWorkspaceId NE ""THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetWorkspaceReleases.p":U).

            RUN rtb/proxy/p/rtbGetWorkspaceReleases.p ON AppServerHandle (
                INPUT-OUTPUT cRowIdent,
                INPUT 0,
                INPUT pcWorkspaceId,
                OUTPUT TABLE-HANDLE hReleaseData).

            IF VALID-HANDLE(hReleaseData) THEN
                lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hReleaseData).
        END.

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hReleaseData NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose: Proxy to return pmod sources for specified workspace.
     Notes:
     Parameters:
    input
      pcWorkspace    - Target workspace
      pcSrcWorkspace - Source workspace (Optional)
      pcProduct      - Source product (Optional)
      pcModule       - Workspace module (Optional)
      plIncExcluded  - TRUE to include EXCluded product modules in results
      plIncPartner   - TRUE to include partner-site-owned product modules

    ------------------------------------------------------------------------------*/

    METHOD PUBLIC LONGCHAR rtbGetWorkspaceSourcePmods(
        INPUT  pcWorkspace    AS CHARACTER,
        INPUT  pcSrcWorkspace AS CHARACTE,
        INPUT  pcProduct      AS CHARACTER,
        INPUT  pcModule       AS CHARACTER,
        INPUT  plIncExcluded  AS LOGICAL,
        INPUT  plIncPartner   AS LOGICAL):

        DEFINE VARIABLE cRowIdent    AS CHARACTER NO-UNDO INITIAL "".
        DEFINE VARIABLE hTT          AS HANDLE    NO-UNDO.
        DEFINE VARIABLE lcJsonResult AS LONGCHAR  NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbGetWorkspaceSourcePmods.p":U).

            RUN rtb/proxy/p/rtbGetWorkspaceSourcePmods.p ON AppServerHandle (
                INPUT-OUTPUT cRowIdent,
                INPUT 0,
                INPUT pcWorkspace,
                INPUT pcSrcWorkspace,  /* Source workspace - optional, leave blank for all */
                INPUT pcProduct,  /* Source product - optional, leave blank for all*/
                INPUT pcModule, /* Source pmod - optional, leave blank for all*/
                INPUT plIncExcluded, /* TRUE to include EXCluded product modules in results */
                INPUT plIncPartner, /* TRUE to include partner-site-owned product modules */
                OUTPUT TABLE-HANDLE hTT).

            IF VALID-HANDLE(hTT) THEN
                lcJsonResult = oTempTableHelper:ConvertTtHandleToJson(hTT).
        END.

        RETURN lcJsonResult.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT hTT NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID rtbMakeDeployment(
        INPUT  pcWorkspace     AS CHARACTER,
        INPUT  pcSiteCode      AS CHARACTER,
        INPUT  piSequence      AS INTEGER):

        DEFINE VARIABLE cError AS CHARACTER NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbMakeDeployment.p":U).

            RUN rtb/proxy/p/rtbMakeDeployment.p ON AppServerHandle(
                INPUT  pcWorkspace,
                INPUT  pcSiteCode,
                INPUT  piSequence,
                OUTPUT cError).

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:  Use the RTB proxies to rename a workspace - *and* all the related data for the workspace
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID rtbRenameWorkspace(
        INPUT  pcOldWorkspaceId AS CHARACTER,
        INPUT  pcNewWorkspaceId AS CHARACTER ):

        DEFINE VARIABLE cError AS CHARACTER NO-UNDO.

        IF pcOldWorkspaceId NE "":U AND
            pcNewWorkspaceId NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbRenameWorkspace.p":U).

            IF pcOldWorkspaceId NE "":U AND
                pcNewWorkspaceId NE "":U THEN
            DO:
                RUN rtb/proxy/p/rtbRenameWorkspace.p ON AppServerHandle (
                    INPUT  pcOldWorkspaceId,
                    INPUT  pcNewWorkspaceId,
                    OUTPUT cError).
            END.

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
        END.
        ELSE
            UNDO, THROW NEW AppError(SUBSTITUTE("The old and new workspace name must be filled in (&1/&2)! Cannot rename workspace",pcOldWorkspaceId, pcNewWorkspaceId), 0).

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID rtbSaveSchemaUpdateList(
        INPUT                 pcWorkspace AS CHARACTER,
        INPUT  DATASET-HANDLE phDsSchupd):

        DEFINE VARIABLE cError AS CHARACTER NO-UNDO.

        IF pcWorkspace NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbSaveSchemaUpdateList.p":U).

            RUN rtb/proxy/p/rtbSaveSchemaUpdateList.p ON AppServerHandle(
                INPUT                 pcWorkspace,
                INPUT  DATASET-HANDLE phDsSchupd,
                OUTPUT                cError).

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT phDsSchupd NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC INTEGER rtbSessionLogin(
        INPUT  pcRTbUser         AS CHARACTER,
        INPUT  pcRTbUserPassword AS CHARACTER):

        DEFINE VARIABLE cError  AS CHARACTER NO-UNDO.
        DEFINE VARIABLE iRtbSid AS INTEGER   NO-UNDO.

        IF pcRTbUser NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbSessionLogin.p":U).

            RUN rtb/proxy/p/rtbSessionLogin.p ON AppServerHandle (
                INPUT  pcRTbUser,
                INPUT  pcRTbUserPassword,
                OUTPUT iRtbSid,
                OUTPUT cError).

            IF (cError NE "":U AND cError NE ?) OR
                iRtbSid = 0  THEN
                UNDO, THROW NEW Progress.Lang.AppError ("Roundtable session login error : " + cError , 0).
        END.

        RETURN iRtbSid.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID rtbSessionLogout(
        INPUT  piRTBSessionID AS INTEGER):

        DEFINE VARIABLE cError AS CHARACTER NO-UNDO.

        IF piRTBSessionID NE 0 THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbSessionLogout.p":U).

            RUN rtb/proxy/p/rtbSessionLogout.p ON AppServerHandle
                (INPUT piRTBSessionID,
                OUTPUT cError).

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError ("Roundtable session logout error : " + cError , 0).
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
     Purpose:
     Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID rtbSetRowValues(
        INPUT                     pcResultSet   AS CHARACTER,
        INPUT-OUTPUT TABLE-HANDLE phTT):

        DEFINE VARIABLE cError AS CHARACTER NO-UNDO.

        IF pcResultSet NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbSetRowValues.p":U).

            RUN rtb/proxy/p/rtbSetRowValues.p ON AppServerHandle (
                INPUT                     pcResultSet,
                INPUT-OUTPUT TABLE-HANDLE phTT,
                OUTPUT cError).

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

        FINALLY:
            DELETE OBJECT phTT NO-ERROR.
        END FINALLY.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose: Use the RTB Proxy rtb/proxy/p/rtbSetSessionProperty.p to set a session property in RTB for a given
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID rtbSetSessionProperty(
        INPUT pcProperty AS CHARACTER,
        INPUT pcValue    AS CHARACTER):

        IF pcProperty NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbSetSessionProperty.p":U).

            RUN rtb/proxy/p/rtbSetSessionProperty.p ON AppServerHandle (
                INPUT  pcProperty,
                INPUT  pcValue).
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

    /*------------------------------------------------------------------------------
            Purpose:
            Notes:
    ------------------------------------------------------------------------------*/

    METHOD PUBLIC VOID rtbVerifyObjectIntegrity(
        INPUT  pcRowid AS CHARACTER):

        DEFINE VARIABLE cError AS CHARACTER NO-UNDO.

        IF pcRowid NE "":U THEN
        DO:
            checkProxyProcedure("rtb/proxy/p/rtbVerifyObjectIntegrity.p").

            RUN rtb/proxy/p/rtbVerifyObjectIntegrity.p ON AppServerHandle(
                INPUT  pcRowid,
                OUTPUT cError).

            IF cError NE "":U THEN
                UNDO, THROW NEW Progress.Lang.AppError(cError, 0).
        END.

        RETURN.

        CATCH err AS Progress.Lang.Error :
            UNDO, THROW err.
        END CATCH.

    END METHOD.

END CLASS.

